﻿function Out-TextToSpeech {

   param (

           [Parameter(Mandatory=$true,

               ValueFromPipeline=$true)]

           [string]

           $Text,

           [int]
           $Speed,

           [string]
           $Voice,

           [switch]

           $Async=$false

         )

   if ($Speed -eq $null) {$Speed = 0}
   $synth.Rate = $Speed
   $synth.SelectVoice($Voice) 

   if ($Async) {

       $synth.SpeakAsync($Text)

   } else {
   
       $synth.Speak($Text)
      

   }

}



function Convert-XAMLtoWindow
{
  param
  (
    [Parameter(Mandatory=$true)]
    [string]
    $XAML,
    
    [string[]]
    $NamedElements,
    
    [switch]
    $PassThru
  )
  
  Add-Type -AssemblyName PresentationFramework
  
  $reader = [System.XML.XMLReader]::Create([System.IO.StringReader]$XAML)
  $result = [System.Windows.Markup.XAMLReader]::Load($reader)
  foreach($Name in $NamedElements)
  {
    $result | Add-Member NoteProperty -Name $Name -Value $result.FindName($Name) -Force
  }
  
  if ($PassThru)
  {
    $result
  }
  else
  {
    $result.ShowDialog()
  }
}

$xaml = @'
<Window
   xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
   xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
   MinHeight="350"
   Width="540"
   SizeToContent="Height"
   Title="Ich sag's!"
   Topmost="False">
   <Grid>
      <Grid.ColumnDefinitions>
         <ColumnDefinition Width="30*"/>
         <ColumnDefinition Width="30*"/>
         <ColumnDefinition Width="30*"/>
      </Grid.ColumnDefinitions>
      <Grid.RowDefinitions>
         <RowDefinition Height="Auto"/>
         <RowDefinition Height="Auto"/>
         <RowDefinition Height="Auto"/>
         <RowDefinition Height="Auto"/>
         <RowDefinition Height="10*"/>
      </Grid.RowDefinitions>
      <TextBlock
         Name="TextMessage"
         Grid.Column="0"
         Grid.Row="0"
         HorizontalAlignment="Left"
         Margin="10,0,0,0"
         FontFamily="Consolas"
         FontSize="20"
         FontWeight="Bold"
         Foreground="Blue">Ich sag's!
      </TextBlock>
      <TextBox
        Name="MeinText"
         Grid.ColumnSpan="3"
         Grid.Row="1"
        Margin="10,10,10,10"
        HorizontalAlignment="Left"
        Width="500"
        Height="100"
        TextWrapping="Wrap">Hurz
      </TextBox>
	    <TextBlock
         Name="SliderLabel"
         Grid.Column="0"
         Grid.Row="2"
         HorizontalAlignment="Right">Geschwindigkeit: 0
      </TextBlock>
      <Slider 
        Name="SliderSpeed"
        Minimum="-10"
        Margin="10,0,10,0"
        Maximum="10"
        IsSnapToTickEnabled="True"
         Grid.Column="1"
         Grid.ColumnSpan="2"
         Grid.Row="2"
        TickFrequency="1">
      </Slider>
      <TextBlock
         Name="LanguageLabel"
         Grid.Column="0"
         Grid.Row="3"
         Margin="10,10,0,0"
         HorizontalAlignment="Right">Stimme
      </TextBlock>
      <ComboBox
      Name="ComboLanguage"
         Grid.Column="1"
         Grid.ColumnSpan="2"
         Grid.Row="3"
	    HorizontalAlignment="Left"
	    Margin="10,10,0,0"
	    Width="200"></ComboBox>
	    <Button
         Name="End"
         Width="80"
         Height="25"
         Grid.Column="1"
         Grid.Row="4"
         Margin="10"
         VerticalAlignment="Bottom">Ende
      </Button>
      <Button
         Name="OK"
         Width="80"
         Height="25"
         Grid.Column="3"
         Grid.Row="4"
         HorizontalAlignment="Right"
         Margin="10"
         VerticalAlignment="Bottom">Sprich
      </Button>
   </Grid>
</Window>
'@

$window = Convert-XAMLtoWindow -XAML $xaml -NamedElements 'TextMessage', 'MeinText', 'LanguageLabel', 'ComboLanguage', 'SliderLabel', 'SliderSpeed', 'End', 'OK' -PassThru


$window.ComboLanguage.add_Loaded(
  {
    [System.Object]$sender = $args[0]
    [System.Windows.RoutedEventArgs]$e = $args[1]   
    # $synth = New-Object System.Speech.Synthesis.SpeechSynthesizer
    $Voices = $synth.GetInstalledVoices().VoiceInfo.Name
    $window.ComboLanguage.ItemsSource = $Voices
    $window.ComboLanguage.SelectedIndex = 0

  }
)




$window.SliderSpeed.add_ValueChanged(
  {
    [System.Object]$sender = $args[0]
    [System.Windows.RoutedPropertyChangedEventArgs`1[[System.Double, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089]]]$e = $args[1]   
    $Speed = $window.SliderSpeed.Value
    $window.SliderLabel.Text = "Geschwindigkeit: $Speed"
  }
)




$window.OK.add_Click(
  {
    [System.Object]$sender = $args[0]
    [System.Windows.RoutedEventArgs]$e = $args[1]   
    $Sprich = $window.MeinText.Text
    $Speed = $window.SliderSpeed.Value
    $Voice = $window.ComboLanguage.SelectedValue
    Out-TextToSpeech -Text $Sprich -Speed $Speed -Voice $Voice
  }
)

$window.End.add_Click(
  {
    [System.Object]$sender = $args[0]
    [System.Windows.RoutedEventArgs]$e = $args[1]   
    $window.Close()
  }
)

[Reflection.Assembly]::LoadWithPartialName('System.Speech') | Out-Null

$synth = New-Object System.Speech.Synthesis.SpeechSynthesizer


$window.ShowDialog()


